@@ -3,13 +3,13 @@ import typing
 # ******************************************************************************
 def split_pair(
                 in_str: str,
-                separator: str,
+                delimiter: str,
                 transform1: callable = lambda x:x,
                 transform2: callable = lambda x:x,
                 default_element: int = 1
 ) -> tuple:
     """
-    split_pair(in_str: str, separator: str, transform1: callable = lambda x:x, transform2: callable = lambda x:x, default_element: int = 1) -> tuple:
+    split_pair(in_str: str, delimiter: str, transform1: callable = lambda x:x, transform2: callable = lambda x:x, default_element: int = 1) -> tuple:
 
     split string into 2 sub strings in any cases:
         '' by '://'                                     => (None,   None)
@@ -21,7 +21,7 @@ def split_pair(
     if not in_str:
         return transform1(None), transform2(None)
 
-    str_parts = in_str.split(separator, 1)
+    str_parts = in_str.split(delimiter, 1)
     if len(str_parts) == 1:
         if default_element:
             # second (right) element is default
@@ -39,9 +39,9 @@ def join_triplets(
     join_triplets(in_list: typing.Union[None, str, tuple, list], level = 0) -> str:
 
     join elements with middle sepataror:
-        [elem1, separator, elem2]   => elem1, separator, elem2
-        [elem1, separator, None]    => elem1
-        [None, separator, elem2]    => elem2
+        [elem1, delimiter, elem2]   => elem1, delimiter, elem2
+        [elem1, delimiter, None]    => elem1
+        [None, delimiter, elem2]    => elem2
     """
     if isinstance(in_list, str) or in_list is None:
         return in_list or ""
@@ -61,7 +61,7 @@ def join_triplets(
                 result = out_list[0] +  (out_list[1] if out_list[0] and out_list[2] else "") + out_list[2]
             else:
                 if not isinstance(in_list[1], (tuple, list)) or len(in_list[1]) not in (1,2):
-                    raise TypeError(f"{type(in_list[1])} '{in_list[1]}' is not correct separator")
+                    raise TypeError(f"{type(in_list[1])} '{in_list[1]}' is not correct delimiter")
                 if len(in_list[1]) == 2 and not in_list[1][0]:
                     result = out_list[0] +  (out_list[1] if out_list[2] else "") + out_list[2]
                 else:
